@@ -76,6 +76,7 @@ from accern_xyme.types import (
     MaintenanceResponse,
     MinimalQueueStatsResponse,
     ModelParamsResponse,
+    ModelReleaseResponse,
     ModelSetupResponse,
     NodeChunk,
     NodeDef,
@@ -91,6 +92,7 @@ from accern_xyme.types import (
     PipelineInfo,
     PipelineInit,
     PipelineList,
+    PipelineReload,
     PutNodeBlob,
     QueueMode,
     QueueStatsResponse,
@@ -113,7 +115,7 @@ else:
     WVD = weakref.WeakValueDictionary
 
 
-__version__ = "0.1.12"
+__version__ = "0.1.14"
 # FIXME: async calls, documentation, auth, summary – time it took etc.
 
 
@@ -760,11 +762,9 @@ class XYMEClient:
         while do_flush():  # we flush until there is nothing to flush anymore
             time.sleep(1.0)
 
-    def get_cache_stats(self, reset: bool = False) -> CacheStats:
+    def get_cache_stats(self) -> CacheStats:
         return cast(CacheStats, self._request_json(
-            METHOD_GET, "/cache_stats", {
-                "reset": int(reset),
-            }))
+            METHOD_GET, "/cache_stats", {}))
 
     def reset_cache(self) -> CacheStats:
         return cast(CacheStats, self._request_json(
@@ -1209,7 +1209,8 @@ class PipelineHandle:
             if success:
                 self.set_dynamic_error_message(None)
 
-    def pretty(self, allow_unicode: bool = True) -> str:
+    def pretty(
+            self, nodes_only: bool = False, allow_unicode: bool = True) -> str:
         nodes = [
             self.get_node(node_id)
             for node_id in sorted(self.get_nodes())
@@ -1220,6 +1221,33 @@ class PipelineHandle:
             NodeHandle,
             List[Tuple[NodeHandle, str, str]],
         ] = collections.defaultdict(list)
+        start_pipe = "├" if allow_unicode else "|"
+        end_pipe = "├" if allow_unicode else "|"
+        before_pipe = "│" if allow_unicode else "|"
+        after_pipe = "│" if allow_unicode else "|"
+        pipe = "┤" if allow_unicode else "|"
+        corner_right = "┐" if allow_unicode else "\\"
+        corner_left = "┘" if allow_unicode else "/"
+        cont_right = "┬" if allow_unicode else "\\"
+        cont_left = "┴" if allow_unicode else "/"
+        cont_skip = "─" if allow_unicode else "-"
+        cont_pipe = "│" if allow_unicode else "|"
+        cont = "┼" if allow_unicode else "-"
+        start_left = "└" if allow_unicode else "\\"
+        bar = "─" if allow_unicode else "-"
+        vsec = "│" if allow_unicode else "|"
+        vstart = "├" if allow_unicode else "|"
+        vend = "┤" if allow_unicode else "|"
+        hsec = "─" if allow_unicode else "-"
+        tl = "┌" if allow_unicode else "+"
+        tr = "┐" if allow_unicode else "+"
+        bl = "└" if allow_unicode else "+"
+        br = "┘" if allow_unicode else "+"
+        conn_top = "┴" if allow_unicode else "-"
+        conn_bottom = "┬" if allow_unicode else "-"
+        space = " "
+        prefix_len = 2 if nodes_only else 3
+        indent = space * prefix_len
 
         def topo(cur: NodeHandle) -> None:
             if cur in already:
@@ -1256,14 +1284,15 @@ class PipelineHandle:
                 in_node, in_key, cur_gap = edge
                 before_gap = cur_gap
                 if in_node == node:
-                    cur_str = f"| {in_key} ({get_in_state(in_node, in_key)}) "
+                    in_state = get_in_state(in_node, in_key)
+                    cur_str = f"{end_pipe} {in_key} ({in_state}) "
                     new_edges.append((None, in_key, cur_gap))
                 else:
-                    cur_str = "|" if in_node is not None else ""
+                    cur_str = before_pipe if in_node is not None else ""
                     cur_gap += gap
                     gap = 0
                     new_edges.append((in_node, in_key, cur_gap))
-                segs.append(f"{' ' * prev_gap}{cur_str}")
+                segs.append(f"{space * prev_gap}{cur_str}")
                 prev_gap = max(0, before_gap - len(cur_str))
             while new_edges:
                 if new_edges[-1][0] is None:
@@ -1281,16 +1310,17 @@ class PipelineHandle:
             prev_gap = 0
             for edge in cur_edges:
                 cur_node, _, cur_gap = edge
-                cur_str = "|" if cur_node is not None else ""
-                segs.append(f"{' ' * prev_gap}{cur_str}")
+                cur_str = after_pipe if cur_node is not None else ""
+                segs.append(f"{space * prev_gap}{cur_str}")
                 new_edges.append(edge)
                 prev_gap = max(0, cur_gap - len(cur_str))
             sout = sorted(
                 outs[node], key=lambda e: order_lookup[e[0]], reverse=True)
             for (in_node, in_key, out_key) in sout:
-                cur_str = f"| {out_key} "
-                end_str = f"| {in_key} ({get_in_state(in_node, in_key)}) "
-                segs.append(f"{' ' * prev_gap}{cur_str}")
+                cur_str = f"{start_pipe} {out_key} "
+                in_state = get_in_state(in_node, in_key)
+                end_str = f"{end_pipe} {in_key} ({in_state}) "
+                segs.append(f"{space * prev_gap}{cur_str}")
                 cur_gap = max(len(cur_str), len(end_str))
                 new_edges.append((in_node, in_key, cur_gap))
                 prev_gap = max(0, cur_gap - len(cur_str))
@@ -1300,29 +1330,172 @@ class PipelineHandle:
             lines: List[str] = []
             edges: List[Tuple[Optional[NodeHandle], str, int]] = []
             for node in order:
-                node_line = \
-                    f"{node.get_short_status(allow_unicode)} " \
-                    f"{node.get_type()}[{node.get_id()}] " \
-                    f"{node.get_highest_chunk()} "
-                total_gap_top = max(
-                    0, sum((edge[2] for edge in edges[:-1])) - len(node_line))
+                top_gaps = [edge[2] for edge in edges[:-1]]
+                top_nodes = [edge[0] for edge in edges]
+                same_ids = [edge[0] == node for edge in edges]
+                empty_top = [edge[0] is None for edge in edges]
                 edges, in_line = draw_in_edges(node, edges)
                 in_line = in_line.rstrip()
                 if in_line:
-                    lines.append(f"  {in_line}")
+                    lines.append(f"{indent}{in_line}")
                 edges, out_line = draw_out_edges(node, edges)
-                total_gap_bottom = max(
-                    0, sum((edge[2] for edge in edges[:-1])) - len(node_line))
-                connector = "\\" if total_gap_bottom > total_gap_top else "/"
+                bottom_gaps = [edge[2] for edge in edges[:-1]]
+                empty_bottom = [edge[0] is None for edge in edges]
+                new_bottom = [
+                    eix >= len(top_nodes) or (
+                        edge[0] is not None and top_nodes[eix] != edge[0]
+                    ) for (eix, edge) in enumerate(edges)
+                ]
+                line_indents: List[str] = []
+                started = False
+                had_same = False
+                highest_iix = -1
+                for (iix, top_gap) in enumerate(top_gaps):
+                    if same_ids[iix]:
+                        had_same = True
+                    if had_same and iix >= len(bottom_gaps):
+                        break
+                    if not line_indents:
+                        line_indents.append(indent)
+                    if empty_top[iix]:
+                        cur_connect = cont_skip if started else space
+                    elif iix >= len(empty_bottom) or empty_bottom[iix]:
+                        if started:
+                            cur_connect = cont_left
+                        else:
+                            cur_connect = start_left
+                        if len(bottom_gaps) < len(top_gaps):
+                            break
+                        started = True
+                    else:
+                        if started:
+                            cur_connect = cont_skip
+                        else:
+                            cur_connect = cont_pipe
+                    cur_line = cont_skip if started else space
+                    gap_size = top_gap - len(cur_connect)
+                    line_indents.append(f"{cur_connect}{cur_line * gap_size}")
+                    highest_iix = iix
+                if line_indents:
+                    line_indents[-1] = line_indents[-1][:-len(indent)]
+                if nodes_only:
+                    mid = f" {node.get_short_status(allow_unicode)} "
+                else:
+                    mid = \
+                        f"{node.get_short_status(allow_unicode)} " \
+                        f"{node.get_name()}({node.get_type()}) " \
+                        f"{node.get_highest_chunk()}"
+                if len(mid) < prefix_len:
+                    mid = f"{mid}{space * (prefix_len - len(mid))}"
+                content = f"{vend if started else vsec}{mid}{vsec}"
+                node_line = f"{''.join(line_indents)}{content}"
+                top_indents: List[str] = []
+                bottom_indents: List[str] = []
+                for iix in range(highest_iix + 1):
+                    top_connect = space if empty_top[iix] else cont_pipe
+                    has_bottom = iix >= len(empty_bottom) or empty_bottom[iix]
+                    bottom_connect = space if has_bottom else cont_pipe
+                    top_gap_size = top_gaps[iix] - len(top_connect)
+                    bottom_gap_size = top_gaps[iix] - len(bottom_connect)
+                    if not top_indents:
+                        top_indents.append(indent)
+                    top_indents.append(f"{top_connect}{space * top_gap_size}")
+                    if not bottom_indents:
+                        bottom_indents.append(indent)
+                    bottom_indents.append(
+                        f"{bottom_connect}{space * bottom_gap_size}")
+                if top_indents:
+                    top_indents[-1] = top_indents[-1][:-len(indent)]
+                if bottom_indents:
+                    bottom_indents[-1] = bottom_indents[-1][:-len(indent)]
+                border_len = len(content) - len(tl) - len(tr)
+                top_border: List[str] = [tl] + [hsec] * border_len + [tr]
+                top_ix = len(indent)
+                for iix in range(highest_iix + 1, len(same_ids)):
+                    if top_ix >= len(top_border):
+                        break
+                    if same_ids[iix]:
+                        top_border[top_ix] = conn_top
+                    if iix >= len(top_gaps):
+                        break
+                    top_ix += top_gaps[iix]
+                bottom_border: List[str] = [bl] + [hsec] * border_len + [br]
+                bottom_ix = len(indent)
+                for iix in range(highest_iix + 1, len(new_bottom)):
+                    if bottom_ix >= len(bottom_border):
+                        break
+                    if new_bottom[iix]:
+                        bottom_border[bottom_ix] = conn_bottom
+                    if iix >= len(bottom_gaps):
+                        break
+                    bottom_ix += bottom_gaps[iix]
+                node_top = f"{''.join(top_indents)}{''.join(top_border)}"
+                node_bottom = \
+                    f"{''.join(bottom_indents)}{''.join(bottom_border)}"
+                total_gap_top = sum(top_gaps) - len(node_line)
+                total_gap_bottom = sum(bottom_gaps) - len(node_line)
+                if total_gap_bottom > total_gap_top:
+                    connector = corner_right
+                    more_gaps = bottom_gaps
+                    top_conn = space
+                    bottom_conn = cont_pipe
+                else:
+                    connector = corner_left
+                    more_gaps = top_gaps
+                    top_conn = cont_pipe
+                    bottom_conn = space
                 if total_gap_bottom == total_gap_top:
-                    connector = "|"
+                    connector = pipe
+                    more_gaps = bottom_gaps
+                    top_conn = cont_pipe
+                    bottom_conn = cont_pipe
                 total_gap = max(total_gap_bottom, total_gap_top)
-                if total_gap > 0:
-                    node_line = f"{node_line}{'-' * total_gap}--{connector}"
-                lines.append(node_line.rstrip())
+                if total_gap >= -prefix_len:
+                    bar_len = total_gap + prefix_len
+                    full_bar = list(bar * bar_len)
+                    full_top = list(space * bar_len)
+                    full_bottom = list(space * bar_len)
+                    bar_ix = prefix_len - len(node_line)
+                    for (before_gap_ix, bar_gap) in enumerate(more_gaps):
+                        bar_ix += bar_gap
+                        if bar_ix < 0:
+                            continue
+                        if bar_ix >= len(full_bar):
+                            break
+                        gap_ix = before_gap_ix + 1
+                        if gap_ix < len(same_ids) and not same_ids[gap_ix]:
+                            mid_connector = cont_skip
+                            mid_top = cont_pipe
+                            mid_bottom = cont_pipe
+                        else:
+                            mid_connector = cont
+                            mid_top = cont_pipe
+                            mid_bottom = cont_pipe
+                        adj_ix = bar_ix - prefix_len
+                        if total_gap_bottom >= adj_ix > total_gap_top:
+                            mid_connector = cont_right
+                            mid_top = space
+                        elif total_gap_bottom < adj_ix <= total_gap_top:
+                            if not empty_top[gap_ix]:
+                                mid_connector = cont_left
+                            else:
+                                mid_top = space
+                            mid_bottom = space
+                        full_bar[bar_ix] = mid_connector
+                        full_top[bar_ix] = mid_top
+                        full_bottom[bar_ix] = mid_bottom
+                    node_line = \
+                        f"{node_line[:-len(vsec)]}{vstart}" \
+                        f"{''.join(full_bar)}{connector}"
+                    node_top = f"{node_top}{''.join(full_top)}{top_conn}"
+                    node_bottom = \
+                        f"{node_bottom}{''.join(full_bottom)}{bottom_conn}"
+                lines.append(node_top.rstrip())
+                lines.append(node_line)
+                lines.append(node_bottom.rstrip())
                 out_line = out_line.rstrip()
                 if out_line:
-                    lines.append(f"  {out_line}")
+                    lines.append(f"{indent}{out_line}")
             return lines
 
         return "\n".join(draw())
@@ -1367,6 +1540,29 @@ class PipelineHandle:
                 warnings_io.flush()
         return res
 
+    def set_attr(
+            self,
+            attr: str,
+            value: Any) -> None:
+        pipe_def = self.get_def()
+        pipe_def[attr] = value  # type: ignore
+        self._client.set_pipeline(self.get_id(), pipe_def)
+
+    def set_name(self, value: str) -> None:
+        self.set_attr("name", value)
+
+    def set_company(self, value: str) -> None:
+        self.set_attr("company", value)
+
+    def set_state(self, value: str) -> None:
+        self.set_attr("state", value)
+
+    def set_high_priority(self, value: bool) -> None:
+        self.set_attr("high_priority", value)
+
+    def set_queue_mng(self, value: Optional[str]) -> None:
+        self.set_attr("queue_mng", value)
+
     def get_visible_blobs(self) -> List[str]:
         return [
             res[0]
@@ -1420,11 +1616,22 @@ class PipelineHandle:
                 "task": None,
             }))["success"]
 
-    def set_kafka_topic_partitions(self, num_partitions: int) -> KafkaTopics:
+    def reload(self, timestamp: Optional[float] = None) -> float:
+        return cast(PipelineReload, self._client._request_json(
+            METHOD_PUT, "/pipeline_reload", {
+                "pipeline": self.get_id(),
+                "timestamp": timestamp,
+            }))["when"]
+
+    def set_kafka_topic_partitions(
+            self,
+            num_partitions: int,
+            large_input_retention: bool = False) -> KafkaTopics:
         return cast(KafkaTopics, self._client._request_json(
             METHOD_POST, "/kafka_topics", {
                 "pipeline": self.get_id(),
                 "num_partitions": num_partitions,
+                "large_input_retention": large_input_retention,
             }))
 
     def post_kafka_objs(self, input_objs: List[Any]) -> List[str]:
@@ -2127,6 +2334,14 @@ class BlobHandle:
             for blob_uri in resp["files"]
         ]
 
+    def convert_model(self) -> ModelReleaseResponse:
+        return cast(ModelReleaseResponse, self._client._request_json(
+            METHOD_POST, "/convert_model", {
+                "blob": self._uri,
+                "pipeline": self.get_pipeline().get_id(),
+            }
+        ))
+
     def __hash__(self) -> int:
         return hash(self.as_str())
 
