@@ -126,6 +126,10 @@ PipelineCreate = TypedDict('PipelineCreate', {
     "nodes": List[str],
     "warnings": List[str],
 })
+PipelineReload = TypedDict('PipelineReload', {
+    "pipeline": str,
+    "when": float,
+})
 NodeDef = TypedDict('NodeDef', {
     "id": str,
     "kind": str,
@@ -304,11 +308,12 @@ DynamicResults = TypedDict('DynamicResults', {
     "results": List[Any],
 })
 CacheStats = TypedDict('CacheStats', {
-    "approx_total": int,
-    "effectiveness": float,
-    "hits": int,
-    "miss": int,
+    "categories": int,
+    "total": int,
 })
 QueueMode = TypedDict('QueueMode', {
     "mode": str,
 })
+ModelReleaseResponse = TypedDict('ModelReleaseResponse', {
+    "release": Optional[str],
+})
